MODULE traadv_mus
   !!======================================================================
   !!                       ***  MODULE  traadv_mus  ***
   !! Ocean  tracers:  horizontal & vertical advective trend
   !!======================================================================
   !! History :       !  2000-06  (A.Estublier)  for passive tracers
   !!                 !  2001-08  (E.Durand, G.Madec)  adapted for T & S
   !!   NEMO     1.0  !  2002-06  (G. Madec)  F90: Free form and module
   !!            3.2  !  2010-05  (C. Ethe, G. Madec)  merge TRC-TRA + switch from velocity to transport
   !!            3.4  !  2012-06  (P. Oddo, M. Vichi) include the upstream where needed
   !!            3.7  !  2015-09  (G. Madec) add the ice-shelf cavities boundary condition
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   tra_adv_mus   : update the tracer trend with the horizontal
   !!                   and vertical advection trends using MUSCL scheme
   !!----------------------------------------------------------------------
   USE oce            ! ocean dynamics and active tracers
   USE trc_oce        ! share passive tracers/Ocean variables
   USE dom_oce        ! ocean space and time domain
   USE trd_oce        ! trends: ocean variables
   USE trdtra         ! tracers trends manager
   USE sbcrnf         ! river runoffs
   USE diaptr         ! poleward transport diagnostics
   USE diaar5         ! AR5 diagnostics

   !
   USE iom            ! XIOS library
   USE in_out_manager ! I/O manager
   USE lib_mpp        ! distribued memory computing
   USE lbclnk         ! ocean lateral boundary condition (or mpp link) 
   USE lib_fortran    ! Fortran utilities (allows no signed zero when 'key_nosignedzero' defined)
   USE halo_mng

   IMPLICIT NONE
   PRIVATE

   PUBLIC   tra_adv_mus   ! routine called by traadv.F90
   
   REAL(wp), ALLOCATABLE, DIMENSION(:,:    ) ::   r1_e1e2t_exh2, r1_e1e2u_exh2, r1_e1e2v_exh2
   REAL(wp), ALLOCATABLE, DIMENSION(:,:    ) ::   rnfmsk_exh2, upsmsk_exh2, mikt_exh2
   REAL(wp), ALLOCATABLE, DIMENSION(:,:,:  ) ::   tmask_exh2, wmask_exh2, umask_exh2, vmask_exh2
   REAL(wp), ALLOCATABLE, DIMENSION(:,:,:  ) ::   e3u_n_exh2, e3v_n_exh2, e3t_n_exh2, e3w_n_exh2
   REAL(wp), ALLOCATABLE, DIMENSION(:,:,:  ) ::   pun_exh2, pvn_exh2, pwn_exh2   ! 3 ocean velocity components
   REAL(wp), ALLOCATABLE, DIMENSION(:,:,:,:) ::   ptb_exh2, pta_exh2        ! before and now tracer fields

   
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:)   ::   upsmsk   !: mixed upstream/centered scheme near some straits
   !                                                           !  and in closed seas (orca 2 and 1 configurations)
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:,:) ::   xind     !: mixed upstream/centered index
   
   LOGICAL  ::   l_trd   ! flag to compute trends
   LOGICAL  ::   l_ptr   ! flag to compute poleward transport
   LOGICAL  ::   l_hst   ! flag to compute heat/salt transport
   
   INTEGER :: jphls = 2

   !! * Substitutions
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OCE 4.0 , NEMO Consortium (2018)
   !! $Id$ 
   !! Software governed by the CeCILL license (see ./LICENSE)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE tra_adv_mus( kt, kit000, cdtype, p2dt, pun, pvn, pwn,             &
      &                                              ptb, pta, kjpt, ld_msc_ups )
      !!----------------------------------------------------------------------
      !!                    ***  ROUTINE tra_adv_mus  ***
      !!
      !! ** Purpose :   Compute the now trend due to total advection of tracers
      !!              using a MUSCL scheme (Monotone Upstream-centered Scheme for
      !!              Conservation Laws) and add it to the general tracer trend.
      !!
      !! ** Method  : MUSCL scheme plus centered scheme at ocean boundaries
      !!              ld_msc_ups=T : 
      !!
      !! ** Action : - update pta  with the now advective tracer trends
      !!             - send trends to trdtra module for further diagnostcs (l_trdtra=T)
      !!             - htr_adv, str_adv : poleward advective heat and salt transport (ln_diaptr=T)
      !!
      !! References : Estubier, A., and M. Levy, Notes Techn. Pole de Modelisation
      !!              IPSL, Sept. 2000 (http://www.lodyc.jussieu.fr/opa)
      !!----------------------------------------------------------------------
      INTEGER                              , INTENT(in   ) ::   kt              ! ocean time-step index
      INTEGER                              , INTENT(in   ) ::   kit000          ! first time step index
      CHARACTER(len=3)                     , INTENT(in   ) ::   cdtype          ! =TRA or TRC (tracer indicator)
      INTEGER                              , INTENT(in   ) ::   kjpt            ! number of tracers
      LOGICAL                              , INTENT(in   ) ::   ld_msc_ups      ! use upstream scheme within muscl
      REAL(wp)                             , INTENT(in   ) ::   p2dt            ! tracer time-step
      REAL(wp), DIMENSION(jpi,jpj,jpk     ), INTENT(in   ) ::   pun, pvn, pwn   ! 3 ocean velocity components
      REAL(wp), DIMENSION(jpi,jpj,jpk,kjpt), INTENT(in   ) ::   ptb             ! before tracer field
      REAL(wp), DIMENSION(jpi,jpj,jpk,kjpt), INTENT(inout) ::   pta             ! tracer trend 
      !
      INTEGER  ::   ji, jj, jk, jn   ! dummy loop indices
      INTEGER  ::   last_khls, ierr             ! local integer
      REAL(wp) ::   zu, z0u, zzwx, zw , zalpha   ! local scalars
      REAL(wp) ::   zv, z0v, zzwy, z0w           !   -      -
      REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::   zwx, zslpx   ! 3D workspace
      REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::   zwy, zslpy   ! -      - 
      !!----------------------------------------------------------------------
      !
 		
	CALL halo_mng_set(jphls)
	
	ALLOCATE(zwx(jplbi:jpi,jplbj:jpj,jpk))
	ALLOCATE(zwy(jplbi:jpi,jplbj:jpj,jpk))
	ALLOCATE(zslpx(jplbi:jpi,jplbj:jpj,jpk))
	ALLOCATE(zslpy(jplbi:jpi,jplbj:jpj,jpk))

	IF (kt==kit000) THEN
		if (.not. allocated(pun_exh2))	ALLOCATE(pun_exh2(jplbi:jpi,jplbj:jpj,jpk))
		if (.not. allocated(pvn_exh2))	ALLOCATE(pvn_exh2(jplbi:jpi,jplbj:jpj,jpk))
		if (.not. allocated(pwn_exh2))	ALLOCATE(pwn_exh2(jplbi:jpi,jplbj:jpj,jpk))
		if (.not. allocated(ptb_exh2))	ALLOCATE(ptb_exh2(jplbi:jpi,jplbj:jpj,jpk,kjpt))
		if (.not. allocated(pta_exh2))	ALLOCATE(pta_exh2(jplbi:jpi,jplbj:jpj,jpk,kjpt))
		if (.not. allocated(r1_e1e2t_exh2))	ALLOCATE(r1_e1e2t_exh2(jplbi:jpi,jplbj:jpj))
		if (.not. allocated(r1_e1e2u_exh2))	ALLOCATE(r1_e1e2u_exh2(jplbi:jpi,jplbj:jpj))
		if (.not. allocated(r1_e1e2v_exh2))	ALLOCATE(r1_e1e2v_exh2(jplbi:jpi,jplbj:jpj))
		if (.not. allocated(tmask_exh2))	ALLOCATE(tmask_exh2(jplbi:jpi,jplbj:jpj,jpk))
		if (.not. allocated(wmask_exh2))	ALLOCATE(wmask_exh2(jplbi:jpi,jplbj:jpj,jpk))
		if (.not. allocated(umask_exh2))	ALLOCATE(umask_exh2(jplbi:jpi,jplbj:jpj,jpk))
		if (.not. allocated(vmask_exh2))	ALLOCATE(vmask_exh2(jplbi:jpi,jplbj:jpj,jpk))
		if (.not. allocated(e3u_n_exh2))	ALLOCATE(e3u_n_exh2(jplbi:jpi,jplbj:jpj,jpk))
		if (.not. allocated(e3v_n_exh2))	ALLOCATE(e3v_n_exh2(jplbi:jpi,jplbj:jpj,jpk))
		if (.not. allocated(e3t_n_exh2))	ALLOCATE(e3t_n_exh2(jplbi:jpi,jplbj:jpj,jpk))
		if (.not. allocated(e3w_n_exh2))	ALLOCATE(e3w_n_exh2(jplbi:jpi,jplbj:jpj,jpk))
		IF( ln_isfcav.and..not.allocated(mikt_exh2)) ALLOCATE(mikt_exh2(jplbi:jpi,jplbj:jpj))
		IF( ld_msc_ups.and..not.allocated(rnfmsk_exh2)) ALLOCATE(rnfmsk_exh2(jplbi:jpi,jplbj:jpj))
		IF( ld_msc_ups.and..not.allocated(upsmsk_exh2)) ALLOCATE(upsmsk_exh2(jplbi:jpi,jplbj:jpj))

		CALL halo_mng_copy(r1_e1e2t, r1_e1e2t_exh2)
		CALL halo_mng_copy(r1_e1e2u, r1_e1e2u_exh2)
		CALL halo_mng_copy(r1_e1e2v, r1_e1e2v_exh2)
		CALL halo_mng_copy(tmask, tmask_exh2)
		CALL halo_mng_copy(wmask, wmask_exh2)
		CALL halo_mng_copy(umask, umask_exh2)
		CALL halo_mng_copy(vmask, vmask_exh2)

		CALL lbc_lnk( 'traadv_mus', r1_e1e2u_exh2, 'U', -1.)
		CALL lbc_lnk( 'traadv_mus', r1_e1e2v_exh2, 'V', -1.)
		CALL lbc_lnk( 'traadv_mus', r1_e1e2t_exh2, 'T', 1.)
		CALL lbc_lnk( 'traadv_mus', tmask_exh2, 'T', 1. )
		CALL lbc_lnk( 'traadv_mus', wmask_exh2, 'W', 1.)
		CALL lbc_lnk( 'traadv_mus', umask_exh2, 'U', 1. )
		CALL lbc_lnk( 'traadv_mus', vmask_exh2, 'V', 1.)
	ENDIF
	
	IF( ln_isfcav ) THEN ; CALL halo_mng_copy(REAL(mikt), mikt_exh2) ; CALL lbc_lnk( 'traadv_mus', mikt_exh2, 'T', 1.) ; ENDIF
	IF( ld_msc_ups) THEN ; CALL halo_mng_copy(rnfmsk, rnfmsk_exh2) ; CALL lbc_lnk( 'traadv_mus', rnfmsk_exh2, 'T', 1.) ; ENDIF
	IF( ld_msc_ups) THEN ; CALL halo_mng_copy(upsmsk, upsmsk_exh2) ; CALL lbc_lnk( 'traadv_mus', upsmsk_exh2, 'T', 1.) ; ENDIF

	CALL halo_mng_copy(e3u_n, e3u_n_exh2)
	CALL halo_mng_copy(e3v_n, e3v_n_exh2)
	CALL halo_mng_copy(e3t_n, e3t_n_exh2)
	CALL halo_mng_copy(e3w_n, e3w_n_exh2)
	CALL halo_mng_copy(pun, pun_exh2)
	CALL halo_mng_copy(pvn, pvn_exh2)
	CALL halo_mng_copy(pwn, pwn_exh2)
	CALL halo_mng_copy(ptb, ptb_exh2)
	CALL halo_mng_copy(pta, pta_exh2)

	CALL lbc_lnk( 'traadv_mus', e3u_n_exh2, 'U', -1., pfillval = 1.0_wp )
	CALL lbc_lnk( 'traadv_mus', e3v_n_exh2, 'V', -1., pfillval = 1.0_wp )
	CALL lbc_lnk( 'traadv_mus', e3t_n_exh2, 'T', 1., pfillval = 1.0_wp )
	CALL lbc_lnk( 'traadv_mus', e3w_n_exh2, 'W', 1., pfillval = 1.0_wp )
	CALL lbc_lnk( 'traadv_mus', pun_exh2, 'U', -1.)
	CALL lbc_lnk( 'traadv_mus', pvn_exh2, 'V', -1.)
	CALL lbc_lnk( 'traadv_mus', pwn_exh2, 'W', 1.)
	CALL lbc_lnk( 'traadv_mus', pta_exh2, 'T', 1.)
	CALL lbc_lnk( 'traadv_mus', ptb_exh2, 'T', 1.)

#     define pun pun_exh2
#     define pvn pvn_exh2
#     define pwn pwn_exh2
#     define ptb ptb_exh2
#     define pta pta_exh2
#     define r1_e1e2t r1_e1e2t_exh2
#     define r1_e1e2u r1_e1e2u_exh2
#     define r1_e1e2v r1_e1e2v_exh2
#     define tmask tmask_exh2
#     define wmask wmask_exh2
#     define umask umask_exh2
#     define vmask vmask_exh2
#     define e3u_n e3u_n_exh2
#     define e3v_n e3v_n_exh2
#     define e3t_n e3t_n_exh2
#     define e3w_n e3w_n_exh2
#     define mikt mikt_exh2
#     define rnfmsk rnfmsk_exh2
#     define upsmsk upsmsk_exh2

      IF( kt == kit000 )  THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) 'tra_adv : MUSCL advection scheme on ', cdtype
         IF(lwp) WRITE(numout,*) '        : mixed up-stream           ', ld_msc_ups
         IF(lwp) WRITE(numout,*) '~~~~~~~'
         IF(lwp) WRITE(numout,*)
         !
         ! Upstream / MUSCL scheme indicator
         !
         ALLOCATE( xind(jplbi:jpi,jplbj:jpj,jpk), STAT=ierr )
         xind(:,:,:) = 1._wp              ! set equal to 1 where up-stream is not needed
         !
         IF( ld_msc_ups ) THEN            ! define the upstream indicator (if asked)
            ALLOCATE( upsmsk(jplbi:jpi,jplbj:jpj), STAT=ierr )
            upsmsk(:,:) = 0._wp                             ! not upstream by default
            !
            DO jk = 1, jpkm1
               xind(:,:,jk) = 1._wp                              &                 ! =>1 where up-stream is not needed
                  &         - MAX ( rnfmsk(:,:) * rnfmsk_z(jk),  &                 ! =>0 near runoff mouths (& closed sea outflows)
                  &                 upsmsk(:,:)                ) * tmask(:,:,jk)   ! =>0 in some user defined area
            END DO
         ENDIF 
         !
      ENDIF 
      !      
      l_trd = .FALSE.
      l_hst = .FALSE.
      l_ptr = .FALSE.
      IF( ( cdtype == 'TRA' .AND. l_trdtra ) .OR. ( cdtype == 'TRC' .AND. l_trdtrc ) )      l_trd = .TRUE.
      IF(   cdtype == 'TRA' .AND. ln_diaptr )                                               l_ptr = .TRUE. 
      IF(   cdtype == 'TRA' .AND. ( iom_use("uadv_heattr") .OR. iom_use("vadv_heattr") .OR. &
         &                          iom_use("uadv_salttr") .OR. iom_use("vadv_salttr")  ) ) l_hst = .TRUE.
      !
      DO jn = 1, kjpt            !==  loop over the tracers  ==!
         !
         !                          !* Horizontal advective fluxes
         !
         !                                !-- first guess of the slopes
         zwx(:,:,jpk) = 0._wp                   ! bottom values
         zwy(:,:,jpk) = 0._wp  
         DO jk = 1, jpkm1                       ! interior values
            DO jj = jplbj, jpj-1      
               DO ji = jplbi, jpi-1   ! vector opt.
                  zwx(ji,jj,jk) = umask(ji,jj,jk) * ( ptb(ji+1,jj,jk,jn) - ptb(ji,jj,jk,jn) )
                  zwy(ji,jj,jk) = vmask(ji,jj,jk) * ( ptb(ji,jj+1,jk,jn) - ptb(ji,jj,jk,jn) )
               END DO
           END DO
         END DO
         ! lateral boundary conditions   (changed sign)
         !CALL lbc_lnk_multi( 'traadv_mus', zwx, 'U', -1. , zwy, 'V', -1. )
         !                                !-- Slopes of tracer
         zslpx(:,:,jpk) = 0._wp                 ! bottom values
         zslpy(:,:,jpk) = 0._wp
         DO jk = 1, jpkm1                       ! interior values
            DO jj = jplbj+1, jpj
               DO ji = jplbi+1, jpi   ! vector opt.
                  zslpx(ji,jj,jk) =                    ( zwx(ji,jj,jk) + zwx(ji-1,jj  ,jk) )   &
                     &            * ( 0.25 + SIGN( 0.25, zwx(ji,jj,jk) * zwx(ji-1,jj  ,jk) ) )
                  zslpy(ji,jj,jk) =                    ( zwy(ji,jj,jk) + zwy(ji  ,jj-1,jk) )   &
                     &            * ( 0.25 + SIGN( 0.25, zwy(ji,jj,jk) * zwy(ji  ,jj-1,jk) ) )
               END DO
            END DO
         END DO
         !
         DO jk = 1, jpkm1                 !-- Slopes limitation
            DO jj = jplbj+1, jpj
               DO ji = jplbi+1, jpi   ! vector opt.
                  zslpx(ji,jj,jk) = SIGN( 1., zslpx(ji,jj,jk) ) * MIN(    ABS( zslpx(ji  ,jj,jk) ),   &
                     &                                                 2.*ABS( zwx  (ji-1,jj,jk) ),   &
                     &                                                 2.*ABS( zwx  (ji  ,jj,jk) ) )
                  zslpy(ji,jj,jk) = SIGN( 1., zslpy(ji,jj,jk) ) * MIN(    ABS( zslpy(ji,jj  ,jk) ),   &
                     &                                                 2.*ABS( zwy  (ji,jj-1,jk) ),   &
                     &                                                 2.*ABS( zwy  (ji,jj  ,jk) ) )
               END DO
           END DO
         END DO
         !
         DO jk = 1, jpkm1                 !-- MUSCL horizontal advective fluxes
            DO jj = jplbj+1, jpj-1
               DO ji = jplbi+1, jpi-1   ! vector opt.
                  ! MUSCL fluxes
                  z0u = SIGN( 0.5, pun(ji,jj,jk) )
                  zalpha = 0.5 - z0u
                  zu  = z0u - 0.5 * pun(ji,jj,jk) * p2dt * r1_e1e2u(ji,jj) / e3u_n(ji,jj,jk)
                  zzwx = ptb(ji+1,jj,jk,jn) + xind(ji,jj,jk) * zu * zslpx(ji+1,jj,jk)
                  zzwy = ptb(ji  ,jj,jk,jn) + xind(ji,jj,jk) * zu * zslpx(ji  ,jj,jk)
                  zwx(ji,jj,jk) = pun(ji,jj,jk) * ( zalpha * zzwx + (1.-zalpha) * zzwy )
                  !
                  z0v = SIGN( 0.5, pvn(ji,jj,jk) )
                  zalpha = 0.5 - z0v
                  zv  = z0v - 0.5 * pvn(ji,jj,jk) * p2dt * r1_e1e2v(ji,jj) / e3v_n(ji,jj,jk)
                  zzwx = ptb(ji,jj+1,jk,jn) + xind(ji,jj,jk) * zv * zslpy(ji,jj+1,jk)
                  zzwy = ptb(ji,jj  ,jk,jn) + xind(ji,jj,jk) * zv * zslpy(ji,jj  ,jk)
                  zwy(ji,jj,jk) = pvn(ji,jj,jk) * ( zalpha * zzwx + (1.-zalpha) * zzwy )
               END DO
            END DO
         END DO
         !CALL lbc_lnk_multi( 'traadv_mus', zwx, 'U', -1. , zwy, 'V', -1. )   ! lateral boundary conditions   (changed sign)
         !
         DO jk = 1, jpkm1                 !-- Tracer advective trend
            DO jj = jplbj+1, jpj-1      
               DO ji = jplbi+1, jpi-1   ! vector opt.
                  pta(ji,jj,jk,jn) = pta(ji,jj,jk,jn) - ( zwx(ji,jj,jk) - zwx(ji-1,jj  ,jk  )       &
                  &                                     + zwy(ji,jj,jk) - zwy(ji  ,jj-1,jk  ) )     &
                  &                                   * r1_e1e2t(ji,jj) / e3t_n(ji,jj,jk)
               END DO
           END DO
         END DO        
         !                                ! trend diagnostics
         IF( l_trd )  THEN
            CALL trd_tra( kt, cdtype, jn, jptra_xad, zwx, pun, ptb(:,:,:,jn) )
            CALL trd_tra( kt, cdtype, jn, jptra_yad, zwy, pvn, ptb(:,:,:,jn) )
         END IF
         !                                 ! "Poleward" heat and salt transports 
         IF( l_ptr )  CALL dia_ptr_hst( jn, 'adv', zwy(:,:,:) )
         !                                 !  heat transport
         IF( l_hst )  CALL dia_ar5_hst( jn, 'adv', zwx(:,:,:), zwy(:,:,:) )
         !
         !                          !* Vertical advective fluxes
         !
         !                                !-- first guess of the slopes
         zwx(:,:, 1 ) = 0._wp                   ! surface & bottom boundary conditions
         zwx(:,:,jpk) = 0._wp
         DO jk = 2, jpkm1                       ! interior values
            zwx(:,:,jk) = tmask(:,:,jk) * ( ptb(:,:,jk-1,jn) - ptb(:,:,jk,jn) )
         END DO
         !                                !-- Slopes of tracer
         zslpx(:,:,1) = 0._wp                   ! surface values
         DO jk = 2, jpkm1                       ! interior value
            DO jj = jplbj, jpj
               DO ji = jplbi, jpi
                  zslpx(ji,jj,jk) =                     ( zwx(ji,jj,jk) + zwx(ji,jj,jk+1) )  &
                     &            * (  0.25 + SIGN( 0.25, zwx(ji,jj,jk) * zwx(ji,jj,jk+1) )  )
               END DO
            END DO
         END DO
         DO jk = 2, jpkm1                 !-- Slopes limitation
            DO jj = jplbj, jpj                      ! interior values
               DO ji = jplbi, jpi
                  zslpx(ji,jj,jk) = SIGN( 1., zslpx(ji,jj,jk) ) * MIN(    ABS( zslpx(ji,jj,jk  ) ),   &
                     &                                                 2.*ABS( zwx  (ji,jj,jk+1) ),   &
                     &                                                 2.*ABS( zwx  (ji,jj,jk  ) )  )
               END DO
            END DO
         END DO
         DO jk = 1, jpk-2                 !-- vertical advective flux
            DO jj = jplbj+1, jpj-1      
               DO ji = jplbi+1, jpi-1   ! vector opt.
                  z0w = SIGN( 0.5, pwn(ji,jj,jk+1) )
                  zalpha = 0.5 + z0w
                  zw  = z0w - 0.5 * pwn(ji,jj,jk+1) * p2dt * r1_e1e2t(ji,jj) / e3w_n(ji,jj,jk+1)
                  zzwx = ptb(ji,jj,jk+1,jn) + xind(ji,jj,jk) * zw * zslpx(ji,jj,jk+1)
                  zzwy = ptb(ji,jj,jk  ,jn) + xind(ji,jj,jk) * zw * zslpx(ji,jj,jk  )
                  zwx(ji,jj,jk+1) = pwn(ji,jj,jk+1) * ( zalpha * zzwx + (1.-zalpha) * zzwy ) * wmask(ji,jj,jk)
               END DO 
            END DO
         END DO
         IF( ln_linssh ) THEN                   ! top values, linear free surface only
            IF( ln_isfcav ) THEN                      ! ice-shelf cavities (top of the ocean)
            DO jj = jplbj, jpj      
               DO ji = jplbi, jpi
                     zwx(ji,jj, mikt(ji,jj) ) = pwn(ji,jj,mikt(ji,jj)) * ptb(ji,jj,mikt(ji,jj),jn)
                  END DO
               END DO   
            ELSE                                      ! no cavities: only at the ocean surface
               zwx(:,:,1) = pwn(:,:,1) * ptb(:,:,1,jn)
            ENDIF
         ENDIF
         !
         DO jk = 1, jpkm1                 !-- vertical advective trend
            DO jj = jplbj+1, jpj-1      
               DO ji = jplbi+1, jpi-1   ! vector opt.
                  pta(ji,jj,jk,jn) =  pta(ji,jj,jk,jn) - ( zwx(ji,jj,jk) - zwx(ji,jj,jk+1) ) * r1_e1e2t(ji,jj) / e3t_n(ji,jj,jk)
               END DO
            END DO
         END DO
         !                                ! send trends for diagnostic
         IF( l_trd )  CALL trd_tra( kt, cdtype, jn, jptra_zad, zwx, pwn, ptb(:,:,:,jn) )
         !
      END DO                     ! end of tracer loop
      !
#     undef pun
#     undef pvn
#     undef pwn
#     undef ptb
#     undef pta
#     undef r1_e1e2t
#     undef r1_e1e2u
#     undef r1_e1e2v
#     undef tmask
#     undef wmask
#     undef umask
#     undef vmask
#     undef e3u_n
#     undef e3v_n
#     undef e3t_n
#     undef e3w_n
#     undef mikt
#     undef rnfmsk
#     undef upsmsk

	CALL halo_mng_copy(pta_exh2, pta)

    last_khls = jphls - ((SIZE(pta_exh2, 1) - SIZE(pta, 1))/2)

    CALL halo_mng_set(last_khls)

	CALL lbc_lnk( 'traadv_mus', pta, 'T', 1. )

	IF( kt==nitend ) THEN
		if (allocated(pun_exh2)) DEALLOCATE(pun_exh2)
    	if (allocated(pvn_exh2)) DEALLOCATE(pvn_exh2)
		if (allocated(pwn_exh2)) DEALLOCATE(pwn_exh2)
	   	if (allocated(ptb_exh2)) DEALLOCATE(ptb_exh2)
		if (allocated(pta_exh2)) DEALLOCATE(pta_exh2)
		if (allocated(r1_e1e2t_exh2)) DEALLOCATE(r1_e1e2t_exh2)
		if (allocated(r1_e1e2u_exh2)) DEALLOCATE(r1_e1e2u_exh2)
		if (allocated(r1_e1e2v_exh2)) DEALLOCATE(r1_e1e2v_exh2)
		if (allocated(tmask_exh2)) DEALLOCATE(tmask_exh2)
		if (allocated(wmask_exh2)) DEALLOCATE(wmask_exh2)
		if (allocated(umask_exh2)) DEALLOCATE(umask_exh2)
		if (allocated(vmask_exh2)) DEALLOCATE(vmask_exh2)
		if (allocated(e3u_n_exh2)) DEALLOCATE(e3u_n_exh2)
		if (allocated(e3v_n_exh2)) DEALLOCATE(e3v_n_exh2)
		if (allocated(e3t_n_exh2)) DEALLOCATE(e3t_n_exh2)
		if (allocated(e3w_n_exh2)) DEALLOCATE(e3w_n_exh2)
       	IF (ln_isfcav.and.allocated(mikt_exh2)) DEALLOCATE(mikt_exh2)
       	IF( ld_msc_ups.and.allocated(rnfmsk_exh2)) DEALLOCATE(rnfmsk_exh2)
		IF( ld_msc_ups.and.allocated(upsmsk_exh2)) DEALLOCATE(upsmsk_exh2)

    ENDIF

	DEALLOCATE(zwx,zwy)
    DEALLOCATE(zslpx,zslpy)
   END SUBROUTINE tra_adv_mus

   !!======================================================================
END MODULE traadv_mus
